@@ -204,7 +204,7 @@ if(Sys.getenv("NO_GPBOOST_ALGO_TESTS") != "NO_GPBOOST_ALGO_TESTS"){
         
         # Training with "wrong" likelihood
         gp_model <- GPModel(group_data = group_data_train, likelihood = "bernoulli_probit",
-                            , matrix_inversion_method = inv_method)
+                            matrix_inversion_method = inv_method)
         expect_error({ 
           bst <- gpboost(data = X_train, label = y_train, gp_model = gp_model,
                          nrounds = 62, learning_rate = 0.01, max_depth = 6,
@@ -288,6 +288,16 @@ if(Sys.getenv("NO_GPBOOST_ALGO_TESTS") != "NO_GPBOOST_ALGO_TESTS"){
         expect_equal(opt_params$best_params$min_data_in_leaf, 10)
         expect_equal(opt_params$best_iter, 7)
         expect_lt(abs(opt_params$best_score-1.224379), TOLERANCE)
+        # Using 'crps_gaussian' as metric
+        opt_params <- gpb.grid.search.tune.parameters(param_grid = param_grid, params = other_params,
+                                                      folds = folds, data = dtrain, gp_model = gp_model,
+                                                      use_gp_model_for_validation=TRUE, verbose_eval = 0,
+                                                      nrounds = 1000, early_stopping_rounds = 10,
+                                                      metric = "crps_gaussian")
+        expect_equal(opt_params$best_params$learning_rate, 0.1)
+        expect_equal(opt_params$best_params$min_data_in_leaf, 10)
+        expect_equal(opt_params$best_iter, 7)
+        expect_lt(abs(opt_params$best_score-0.4622949), TOLERANCE)
         
         # Parameter tuning with 'tune.pars.bayesian.optimization'
         if(inv_method=="cholesky"){
@@ -431,14 +441,14 @@ if(Sys.getenv("NO_GPBOOST_ALGO_TESTS") != "NO_GPBOOST_ALGO_TESTS"){
         
         # Newton updates for tree leaves
         if(inv_method=="cholesky"){
-          params <- list(learning_rate = 0.1,
+          params_loc <- list(learning_rate = 0.1,
                          max_depth = 6,
                          min_data_in_leaf = 5,
                          objective = "regression_l2",
                          leaves_newton_update = TRUE)
-          gp_model <- GPModel(group_data = group_data_train)
+          gp_model <- GPModel(group_data = group_data_train, matrix_inversion_method = inv_method)
           gp_model$set_optim_params(params=params_gp)
-          cvbst <- gpb.cv(params = params,
+          cvbst <- gpb.cv(params = params_loc,
                           data = dtrain,
                           gp_model = gp_model,
                           nrounds = 100,
@@ -529,7 +539,7 @@ if(Sys.getenv("NO_GPBOOST_ALGO_TESTS") != "NO_GPBOOST_ALGO_TESTS"){
         expect_lt(abs(bst$best_score - 3.058637),TOLERANCE)
         # CV
         if(inv_method=="cholesky"){
-          gp_model <- GPModel(group_data = group_data_train)
+          gp_model <- GPModel(group_data = group_data_train, matrix_inversion_method = inv_method)
           gp_model$set_optim_params(params=params_gp)
           cvbst <- gpb.cv(params = params,
                           data = dtrain,
@@ -542,8 +552,8 @@ if(Sys.getenv("NO_GPBOOST_ALGO_TESTS") != "NO_GPBOOST_ALGO_TESTS"){
                           folds = folds,
                           verbose = 0,
                           eval = l4_loss, metric = "l4")
-          expect_equal(cvbst$best_iter, 52)
-          expect_lt(abs(cvbst$best_score - 2.932338),TOLERANCE2)
+          expect_equal(cvbst$best_iter, 59)
+          expect_lt(abs(cvbst$best_score - 2.983831),TOLERANCE2)
         }
         
         # Use of validation data and test_neg_log_likelihood as metric
@@ -587,6 +597,34 @@ if(Sys.getenv("NO_GPBOOST_ALGO_TESTS") != "NO_GPBOOST_ALGO_TESTS"){
         nll <- 0.5 * mean((y_test - pred)^2 / var_est + log(var_est * 2 * pi))
         expect_lt(abs(bst$best_score - nll),TOLERANCE)
         
+        # Use of validation data and crps_gaussian as metric
+        gp_model <- GPModel(group_data = group_data_train, matrix_inversion_method = inv_method)
+        set_prediction_data(gp_model, group_data_pred = group_data_test)
+        set_optim_params(gp_model, params=params_gp)
+        bst <- gpb.train(data = dtrain, gp_model = gp_model, nrounds = 10,
+                         learning_rate = 0.01, max_depth = 6, min_data_in_leaf = 5,
+                         objective = "regression_l2", verbose = 0,
+                         valids = valids, early_stopping_rounds = 5,
+                         use_gp_model_for_validation = TRUE, metric = "crps_gaussian")
+        expect_equal(bst$best_iter, 10)
+        pred <- predict(bst, data = X_test, group_data_pred = group_data_test, 
+                        pred_latent = FALSE, predict_var = TRUE)
+        resid <- (y_test - pred[['response_mean']]) / sqrt(pred[['response_var']])
+        crps <- mean(sqrt(pred[['response_var']]) * (-1/sqrt(pi) + 2 * dnorm(resid) + resid * (2 * pnorm(resid) - 1)))
+        expect_lt(abs(bst$best_score - crps),TOLERANCE)
+        # Use of validation data and crps_gaussian as metric without a GPModel
+        bst <- gpb.train(data = dtrain, nrounds = 10, learning_rate = 0.01, max_depth = 6, min_data_in_leaf = 5,
+                         objective = "regression_l2", verbose = 0,
+                         valids = valids, early_stopping_rounds = 5,
+                         metric = "crps_gaussian")
+        expect_equal(bst$best_iter, 10)
+        predtrain <- predict(bst, data = X_train, pred_latent = TRUE)
+        var_est <- var(y_train - predtrain)
+        pred <- predict(bst, data = X_test, pred_latent = TRUE)
+        resid <- (y_test - pred) / sqrt(var_est)
+        crps <- mean(sqrt(var_est) * (-1/sqrt(pi) + 2 * dnorm(resid) + resid * (2 * pnorm(resid) - 1)))
+        expect_lt(abs(bst$best_score - crps),TOLERANCE)
+        
         ## Cannot have NA's in response variable
         expect_error({
           gp_model <- GPModel(group_data = group_data_train)
@@ -1430,7 +1468,7 @@ if(Sys.getenv("NO_GPBOOST_ALGO_TESTS") != "NO_GPBOOST_ALGO_TESTS"){
       filename_save_raw_data <- tempfile(fileext = ".model")
       gpb.save(bst, filename=filename_save_raw_data, save_raw_data = TRUE)
       # finalize and destroy models
-      bst$finalize()
+      bst$.__enclos_env__$private$finalize()
       expect_null(bst$.__enclos_env__$private$handle)
       rm(bst)
       rm(gp_model)
@@ -1482,7 +1520,7 @@ if(Sys.getenv("NO_GPBOOST_ALGO_TESTS") != "NO_GPBOOST_ALGO_TESTS"){
       filename <- tempfile(fileext = ".model")
       gpb.save(bst, filename=filename, save_raw_data = FALSE)
       # finalize and destroy models
-      bst$finalize()
+      bst$.__enclos_env__$private$finalize()
       expect_null(bst$.__enclos_env__$private$handle)
       rm(bst)
       rm(gp_model)
@@ -1514,7 +1552,7 @@ if(Sys.getenv("NO_GPBOOST_ALGO_TESTS") != "NO_GPBOOST_ALGO_TESTS"){
                                                    start_iteration = start_iteration)
       model_str_raw_data <- bst$save_model_to_string(save_raw_data = TRUE)
       # finalize and destroy models
-      bst$finalize()
+      bst$.__enclos_env__$private$finalize()
       expect_null(bst$.__enclos_env__$private$handle)
       rm(bst)
       rm(gp_model)
